package com.qmth.themis.business.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.themis.business.bean.backend.OpenExamBean;
import com.qmth.themis.business.bean.backend.OpenExamCourseBean;
import com.qmth.themis.business.bean.backend.OpenRecordAnswerBean;
import com.qmth.themis.business.bean.backend.OpenRecordNeedMarkBean;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamPaperCacheBean;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TEExam;
import com.qmth.themis.business.entity.TEExamCourse;
import com.qmth.themis.business.enums.InvigilateMonitorStatusEnum;
import com.qmth.themis.business.enums.RecordSelectStrategyEnum;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.FileUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class TEOpenServiceImpl implements TEOpenService {

    private final static Logger log = LoggerFactory.getLogger(TEOpenServiceImpl.class);

    @Resource
    TEExamService examService;

    @Resource
    TEExamCourseService examCourseService;

    @Resource
    TEExamPaperService examPaperService;

    @Resource
    TOeExamRecordService examRecordService;

    @Resource
    TOeExamAnswerService examAnswerService;

    @Resource
    TEExamStudentService examStudentService;

    @Resource
    OssUtil ossUtil;

    @Override
    public List<OpenExamBean> examQueryPage(Long examId, String examCode, int pageNumber, int pageSize) {
        Page<OpenExamBean> ipage = new Page<>(pageNumber, pageSize);
        ipage.addOrder(OrderItem.desc("t.id"));
        IPage<OpenExamBean> ret = examService.examQueryForOpen(ipage, examId, examCode);
        return ret.getRecords();
    }

    @Override
    public List<OpenExamCourseBean> examCourseQueryPage(Long examId, String courseCode, Boolean hasPaper,
            int pageNumber, int pageSize) {
        Page<OpenExamCourseBean> ipage = new Page<>(pageNumber, pageSize);
        ipage.addOrder(OrderItem.desc("t.id"));
        IPage<OpenExamCourseBean> ret = examCourseService.examCourseQueryForOpen(ipage, examId, courseCode, hasPaper);
        return ret.getRecords();
    }

    @Override
    public JSONObject examPaperDetail(Long paperId, String filter) {
        String tempDir = SystemConstant.TEMP_FILES_DIR;
        String dir = tempDir + "/" + uuid() + "/";
        File dfile = new File(dir);

        try {
            ExamPaperCacheBean paper = examPaperService.getExamPaperCacheBean(paperId);
            if (paper == null) {
                throw new BusinessException("未找到试卷信息");
            }
            checkExam(paper.getExamId());
            dfile.mkdirs();
            String paperPath = paper.getPaperViewPath();
            String anwserPath = paper.getAnswerPath();
            File paperFile = new File(dir + uuid() + ".json");
            File anwserFile = new File(dir + uuid() + ".json");
            FileUtil.saveUrlAsFile(ossUtil.getAliYunOssPrivateDomain().getPrivateUrl() + "/" + paperPath, paperFile);
            FileUtil.saveUrlAsFile(ossUtil.getAliYunOssPrivateDomain().getPrivateUrl() + "/" + anwserPath, anwserFile);
            JSONObject answerJson = JSONObject.parseObject(FileUtil.readFileContent(anwserFile));
            JSONArray answerdetails = answerJson.getJSONArray("details");
            JSONObject structJson = JSONObject.parseObject(FileUtil.readFileContent(paperFile));
            structJson.put("id", paperId);
            structJson.put("code", paper.getCode());
            JSONArray structdetails = structJson.getJSONArray("details");
            for (int i = 0; i < answerdetails.size(); i++) {
                JSONArray answerdetailquestions = answerdetails.getJSONObject(i).getJSONArray("questions");
                JSONArray structdetailquestions = structdetails.getJSONObject(i).getJSONArray("questions");
                for (int j = 0; j < structdetailquestions.size(); j++) {
                    JSONObject answerquestion = answerdetailquestions.getJSONObject(j);
                    JSONObject structquestion = structdetailquestions.getJSONObject(j);
                    if ("objective".equals(filter)) {
                        if (structquestion.getInteger("structType").intValue() == 1
                                || structquestion.getInteger("structType").intValue() == 2
                                || structquestion.getInteger("structType").intValue() == 3) {
                            if (structquestion.getInteger("structType").intValue() == 3) {
                                structquestion.put("answer", answerquestion.getBoolean("answer"));
                            } else {
                                structquestion.put("answer", answerquestion.getJSONArray("answer"));
                            }
                        } else {
                            if (structquestion.getInteger("structType").intValue() == 6) {
                                JSONArray answersubQuestions = answerquestion.getJSONArray("subQuestions");
                                JSONArray structsubQuestions = structquestion.getJSONArray("subQuestions");
                                for (int k = 0; k < structsubQuestions.size(); k++) {
                                    JSONObject answersubquestion = answersubQuestions.getJSONObject(k);
                                    JSONObject structsubquestion = structsubQuestions.getJSONObject(k);
                                    if (structsubquestion.getInteger("structType").intValue() == 1
                                            || structsubquestion.getInteger("structType").intValue() == 2
                                            || structsubquestion.getInteger("structType").intValue() == 3) {
                                        if (structsubquestion.getInteger("structType").intValue() == 3) {
                                            structsubquestion.put("answer", answersubquestion.getBoolean("answer"));
                                        } else {
                                            structsubquestion.put("answer", answersubquestion.getJSONArray("answer"));
                                        }
                                    } else {
                                        structsubQuestions.remove(k);
                                        k--;
                                    }
                                }
                            } else {
                                structdetailquestions.remove(j);
                                j--;
                            }
                        }

                    } else if ("subjective".equals(filter)) {
                        if (structquestion.getInteger("structType").intValue() != 1
                                && structquestion.getInteger("structType").intValue() != 2
                                && structquestion.getInteger("structType").intValue() != 3) {
                            structquestion.put("answer", answerquestion.getJSONArray("answer"));
                        } else {
                            if (structquestion.getInteger("structType").intValue() == 6) {
                                JSONArray answersubQuestions = answerquestion.getJSONArray("subQuestions");
                                JSONArray structsubQuestions = structquestion.getJSONArray("subQuestions");
                                for (int k = 0; k < structsubQuestions.size(); k++) {
                                    JSONObject answersubquestion = answersubQuestions.getJSONObject(k);
                                    JSONObject structsubquestion = structsubQuestions.getJSONObject(k);
                                    if (structsubquestion.getInteger("structType").intValue() != 1
                                            && structsubquestion.getInteger("structType").intValue() != 2
                                            && structsubquestion.getInteger("structType").intValue() != 3) {
                                        structsubquestion.put("answer", answersubquestion.getJSONArray("answer"));
                                    } else {
                                        structsubQuestions.remove(k);
                                        k--;
                                    }
                                }
                            } else {
                                structdetailquestions.remove(j);
                                j--;
                            }
                        }
                    } else {
                        if (structquestion.getInteger("structType").intValue() != 6) {
                            if (structquestion.getInteger("structType").intValue() == 3) {
                                structquestion.put("answer", answerquestion.getBoolean("answer"));
                            } else {
                                structquestion.put("answer", answerquestion.getJSONArray("answer"));
                            }
                        } else {
                            JSONArray answersubQuestions = answerquestion.getJSONArray("subQuestions");
                            JSONArray structsubQuestions = structquestion.getJSONArray("subQuestions");
                            for (int k = 0; k < structsubQuestions.size(); k++) {
                                JSONObject answersubquestion = answersubQuestions.getJSONObject(k);
                                JSONObject structsubquestion = structsubQuestions.getJSONObject(k);
                                if (structsubquestion.getInteger("structType").intValue() == 3) {
                                    structsubquestion.put("answer", answersubquestion.getBoolean("answer"));
                                } else {
                                    structsubquestion.put("answer", answersubquestion.getJSONArray("answer"));
                                }
                            }
                        }
                    }

                }
            }
            return structJson;
        } finally {
            FileUtil.deleteFolder(dir);
        }
    }

    private String uuid() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    private void checkExam(Long examId) {
        TEExam exam = examService.getById(examId);
        if (exam == null) {
            throw new BusinessException("未找到考试信息");
        }
        if (exam.getEnable() == null || exam.getEnable().intValue() == 0) {
            throw new BusinessException("考试批次已禁用");
        }
        if (!InvigilateMonitorStatusEnum.FINISHED.equals(exam.getMonitorStatus())) {
            throw new BusinessException("考试批次监考未结束");
        }
        List<TEExamCourse> courses = examCourseService.findByExamId(examId);
        if (courses == null || courses.size() == 0) {
            throw new BusinessException("考试批次下没有课程");
        }
        for (TEExamCourse course : courses) {
            if (course.getHasAnswer() == null || course.getHasAnswer().intValue() == 0) {
                throw new BusinessException("考试批次下标答未齐全");
            }
        }
    }

    @Override
    public JSONArray examRecordNeedMark(Long examId, String courseCode, Long idGt, Integer count) {
        checkExam(examId);
        List<OpenRecordNeedMarkBean> list;
        for (; ; ) {
            List<Long> examStudentIds = examStudentService.findExamStudentIdsNeedMark(examId, courseCode, idGt, count);
            if (examStudentIds == null || examStudentIds.size() == 0) {
                return null;
            }
            list = findExamRecordNeedMark(examId, examStudentIds);
            if (list == null || list.size() == 0) {
                idGt = examStudentIds.get(examStudentIds.size() - 1);
            } else {
                break;
            }
        }
        List<Long> ids = list.stream().map(dto -> dto.getExamRecordId()).collect(Collectors.toList());
        List<OpenRecordAnswerBean> answers = examAnswerService.findByRecordIds(ids);
        if (answers != null && answers.size() > 0) {
            for (int i = 0; i < answers.size(); i++) {//剔除客观题答案
                OpenRecordAnswerBean answer = answers.get(i);
                Map<String, Integer> struct = examPaperService.getPaperStructCacheBean(answer.getPaperId());
                Integer type = struct.get(RedisKeyHelper
                        .examAnswerHashKey(answer.getMainNumber(), answer.getSubNumber(), answer.getSubIndex()));
                if (type == 1 || type == 2 || type == 3) {
                    answers.remove(i);
                    i--;
                }
            }
        }
        if (answers != null && answers.size() > 0) {
            Map<Long, Integer> map = new HashMap<>();
            for (int i = 0; i < list.size(); i++) {
                map.put(list.get(i).getExamRecordId(), i);
            }
            for (OpenRecordAnswerBean anwser : answers) {
                OpenRecordNeedMarkBean mark = list.get(map.get(anwser.getRecordId()));
                List<OpenRecordAnswerBean> ans = mark.getAnswers();
                if (ans == null) {
                    ans = new ArrayList<OpenRecordAnswerBean>();
                    mark.setAnswers(ans);
                }
                ans.add(anwser);
            }
        }
        JSONArray ja = JSONArray.parseArray(JSONArray.toJSONString(list));
        for (int i = 0; i < ja.size(); i++) {
            JSONObject job = ja.getJSONObject(i);
            JSONArray answersJa = job.getJSONArray("answers");
            if (answersJa != null) {
                for (int j = 0; j < answersJa.size(); j++) {
                    JSONObject answersJob = answersJa.getJSONObject(j);
                    answersJob.remove("recordId");
                    answersJob.remove("paperId");
                    String answerStr = answersJob.getString("answer");
                    if (StringUtils.isNotBlank(answerStr)) {
                        if (answerStr.toLowerCase().startsWith("true") || answerStr.toLowerCase().startsWith("false")) {
                            answersJob.put("answer", Boolean.getBoolean(answerStr.replaceAll("\n", "")));
                        } else {
                            answersJob.put("answer", answersJob.getJSONArray("answer"));
                        }
                    }
                }
            }
        }
        return ja;
    }

    private List<OpenRecordNeedMarkBean> findExamRecordNeedMark(Long examId, List<Long> examStudentIds) {
        List<OpenRecordNeedMarkBean> list = examRecordService.findExamRecordNeedMark(examId, examStudentIds);
        if (list == null || list.size() == 0) {
            return null;
        }
        for (int i = 0; i < list.size(); i++) {//剔除没有主观题的试卷考试记录
            ExamPaperCacheBean paper = examPaperService.getExamPaperCacheBean(list.get(i).getPaperId());
            if (paper.getTotalSubjectiveScore() == 0) {
                list.remove(i);
                i--;
            }
        }
        if (list == null || list.size() == 0) {
            return null;
        }
        TEExam exam = examService.getById(examId);
        if (RecordSelectStrategyEnum.HIGHEST_TOTAL_SCORE.equals(exam.getRecordSelectStrategy())) {//全阅
            return list;
        } else if (RecordSelectStrategyEnum.HIGHEST_OBJECTIVE_SCORE
                .equals(exam.getRecordSelectStrategy())) {//客观分最高，相同则都阅
            Map<Long, List<OpenRecordNeedMarkBean>> map = new HashMap<>();
            for (OpenRecordNeedMarkBean bean : list) {
                List<OpenRecordNeedMarkBean> temList = map.get(bean.getExamStudentId());
                if (temList == null) {
                    temList = new ArrayList<OpenRecordNeedMarkBean>();
                    temList.add(bean);
                    map.put(bean.getExamStudentId(), temList);
                } else {
                    if (temList.get(0).getObjectiveScore() == bean.getObjectiveScore()) {
                        temList.add(bean);
                    } else if (temList.get(0).getObjectiveScore() < bean.getObjectiveScore()) {
                        temList = new ArrayList<OpenRecordNeedMarkBean>();
                        temList.add(bean);
                        map.put(bean.getExamStudentId(), temList);
                    }
                }
            }
            List<OpenRecordNeedMarkBean> ret = new ArrayList<OpenRecordNeedMarkBean>();
            for (List<OpenRecordNeedMarkBean> tem : map.values()) {
                for (OpenRecordNeedMarkBean bean : tem) {
                    ret.add(bean);
                }
            }
            Collections.sort(ret, new Comparator<OpenRecordNeedMarkBean>() {

                @Override
                public int compare(OpenRecordNeedMarkBean o1, OpenRecordNeedMarkBean o2) {
                    Long c1 = o1.getExamStudentId();
                    Long c2 = o2.getExamStudentId();
                    if (c1 > c2) {
                        return 1;
                    } else if (c1 < c2) {
                        return -1;
                    } else {
                        return 0;
                    }
                }

            });
            return ret;
        } else if (RecordSelectStrategyEnum.LATEST.equals(exam.getRecordSelectStrategy())) {//最后一次提交
            Map<Long, OpenRecordNeedMarkBean> map = new HashMap<>();
            for (OpenRecordNeedMarkBean bean : list) {
                OpenRecordNeedMarkBean old = map.get(bean.getExamStudentId());
                if (old == null || old.getFinishTime() < bean.getFinishTime()) {
                    map.put(bean.getExamStudentId(), bean);
                }
            }
            List<OpenRecordNeedMarkBean> ret = map.values().stream().collect(Collectors.toList());
            Collections.sort(ret, new Comparator<OpenRecordNeedMarkBean>() {

                @Override
                public int compare(OpenRecordNeedMarkBean o1, OpenRecordNeedMarkBean o2) {
                    Long c1 = o1.getExamStudentId();
                    Long c2 = o2.getExamStudentId();
                    if (c1 > c2) {
                        return 1;
                    } else if (c1 < c2) {
                        return -1;
                    } else {
                        return 0;
                    }
                }

            });
            return ret;
        } else {
            throw new BusinessException("考试批次阅卷类型未定义");
        }
    }
}
