package com.qmth.themis.business.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.bean.backend.*;
import com.qmth.themis.business.cache.ExamActivityRecordCacheUtil;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.*;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dao.TOeExamRecordMapper;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.dto.response.MarkResultDto;
import com.qmth.themis.business.dto.response.TEExamUnFinishDto;
import com.qmth.themis.business.entity.TOeExamAnswer;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.UidUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.FileUtil;
import com.qmth.themis.common.util.SimpleBeanUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;

/**
 * @Description: 考试记录 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TOeExamRecordServiceImpl extends ServiceImpl<TOeExamRecordMapper, TOeExamRecord>
        implements TOeExamRecordService {

    private final static Logger log = LoggerFactory.getLogger(TOeExamRecordServiceImpl.class);

    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd");

    @Resource
    MqDtoService mqDtoService;

    @Resource
    TOeExamRecordMapper tOeExamRecordMapper;

    @Resource
    RedisUtil redisUtil;

    @Resource
    TEExamPaperService examPaperService;

    @Resource
    TEExamService examService;

    @Resource
    TOeExamAnswerService examAnswerService;

    @Resource
    TEExamActivityService examActivityService;

    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    OssUtil ossUtil;

    @Resource
    UidUtil uidUtil;

    /**
     * 获取考试未完列表
     *
     * @param studentId
     * @param examId
     * @param orgId
     * @return
     */
    @Override
    public Map getUnFinishExam(Long studentId, Long examId, Long orgId) {
        TEExamUnFinishDto teExamUnFinishDto = tOeExamRecordMapper.getUnFinishExam(studentId, examId, orgId);
        if (Objects.nonNull(teExamUnFinishDto)) {
            Map finalMap = new HashMap();
            Map<String, Object> waitingMap = new HashMap();
            waitingMap.put("id", teExamUnFinishDto.getId());
            waitingMap.put("examActivityId", teExamUnFinishDto.getExamActivityId());
            waitingMap.put("name", teExamUnFinishDto.getName());
            waitingMap.put("mode", teExamUnFinishDto.getMode());
            waitingMap.put("preNotice", teExamUnFinishDto.getPreNotice());
            waitingMap.put("preNoticeStaySeconds", teExamUnFinishDto.getPreNoticeStaySeconds());
            waitingMap.put("postNotice", teExamUnFinishDto.getPostNotice());
            finalMap.put("waiting", waitingMap);

            Map<String, Object> activityMap = new HashMap();
            activityMap.put("recordId", teExamUnFinishDto.getRecordId());
            activityMap.put("minDurationSeconds", teExamUnFinishDto.getMinDurationSeconds());
            activityMap.put("cameraPhotoUpload", teExamUnFinishDto.getCameraPhotoUpload());
            activityMap.put("courseCode", teExamUnFinishDto.getCourseCode());
            activityMap.put("examStudentId", teExamUnFinishDto.getExamStudentId());
            activityMap.put("inProcessLivenessVerify", teExamUnFinishDto.getInProcessLivenessVerify());
            activityMap.put("inProcessRealnessVerify", teExamUnFinishDto.getInProcessRealnessVerify());
            activityMap.put("inProcessFaceStrangerIgnore", teExamUnFinishDto.getInProcessFaceStrangerIgnore());
            activityMap.put("inProcessFaceVerify", teExamUnFinishDto.getInProcessFaceVerify());
            activityMap.put("courseName", teExamUnFinishDto.getCourseName());
            activityMap.put("inProcessLivenessFixedRange", teExamUnFinishDto.getInProcessLivenessFixedRange());
            activityMap.put("prepareSeconds", teExamUnFinishDto.getPrepareSeconds());
            activityMap.put("fouceFinish", teExamUnFinishDto.getFouceFinish());
            activityMap.put("entryAuthenticationPolicy", teExamUnFinishDto.getEntryAuthenticationPolicy());
            activityMap.put("maxDurationSeconds", teExamUnFinishDto.getMaxDurationSeconds());
            activityMap.put("examCount", teExamUnFinishDto.getExamCount());
            activityMap.put("inProcessLivenessJudgePolicy", teExamUnFinishDto.getInProcessLivenessJudgePolicy());
            activityMap.put("id", teExamUnFinishDto.getExamActivityId());

            finalMap.put("activity", activityMap);
            return finalMap;
        } else {
            return null;
        }
    }

    @Transactional
    @Override
    public Long saveByPrepare(Long examId, Long examActivityId, Long examStudentId, Long paperId,
            Integer serialNumber) {
        ExamActivityCacheBean ac = examActivityService.getExamActivityCacheBean(examActivityId);
        ExamCacheBean exam = examService.getExamCacheBean(examId);
        TOeExamRecord er = new TOeExamRecord();
        er.setId(uidUtil.getId());
        er.setExamId(examId);
        er.setExamActivityId(examActivityId);
        er.setExamStudentId(examStudentId);
        er.setPaperId(paperId);
        er.setSerialNumber(serialNumber);
        er.setFirstPrepareTime(System.currentTimeMillis());
        er.setStatus(ExamRecordStatusEnum.FIRST_PREPARE);
        er.setObjectiveScore(0.0);
        er.setMonitorKey(String.valueOf(redisUtil.getRedisSequence()));
        er.setAlreadyBreakCount(0);
        er.setStartTime(ac.getStartTime());
        er.setEndTime(ac.getFinishTime());
        er.setOpeningSeconds(ac.getOpeningSeconds());
        er.setMinDurationSeconds(exam.getMinDurationSeconds());
        er.setMaxDurationSeconds(ac.getMaxDurationSeconds());
        er.setForceFinish(exam.getForceFinish());
        ExamPaperCacheBean ep = examPaperService.getExamPaperCacheBean(paperId);
        if (StringUtils.isBlank(ep.getAnswerPath())) {
            er.setHasAnswerFile(0);
        } else {
            er.setHasAnswerFile(1);
        }
        Map<String, Object> map = SimpleBeanUtil.objectToMap(er);
        redisUtil.setForHash(RedisKeyHelper.examRecordCacheKey(er.getId()), map);
        dataInitMq(map);
        return er.getId();
    }

    /**
     * 计算客观分
     */
    @Override
    public void calculateObjectiveScore(Map<String, Object> param) {
        Long recordId = (Long) param.get("recordId");
        Integer mainNumber = (Integer) param.get("mainNumber");
        Integer subNumber = (Integer) param.get("subNumber");
        Integer subIndex = (Integer) param.get("subIndex");
        Long paperId = ExamRecordCacheUtil.getPaperId(recordId);
        String key = RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex);

        //考生作答缓存
        ExamStudentAnswerCacheBean answer = (ExamStudentAnswerCacheBean) redisUtil
                .get(RedisKeyHelper.examAnswerKey(recordId), key);
        if (answer == null) {
            log.error("no ExamStudentAnswerCacheBean for calculateObjectiveScore recordId:" + recordId + " key:" + key);

            // 计算客观分总分
            calculateTotalObjectiveScore(param);
            return;
        }
        //整卷客观题标答缓存集合
        Map<String, ObjectiveAnswerCacheBean> map = examPaperService.getObjectiveAnswerCacheBean(paperId);
        if (map == null || map.size() == 0) {
            log.info("no ObjectiveAnswerCacheBean map for calculateObjectiveScore recordId:" + recordId + " paperId:"
                    + paperId);
            // 更新分数
            answer.setScore(0.0);
            redisUtil.set(RedisKeyHelper.examAnswerKey(recordId), key, answer);

            // 计算客观分总分
            calculateTotalObjectiveScore(param);
            return;
        }
        //客观题标答缓存
        ObjectiveAnswerCacheBean cb = map.get(key);
        if (cb == null) {
            log.info("no ObjectiveAnswerCacheBean for calculateObjectiveScore recordId:" + recordId + " key:" + key);
            // 更新分数
            answer.setScore(0.0);
            redisUtil.set(RedisKeyHelper.examAnswerKey(recordId), key, answer);

            // 计算客观分总分
            calculateTotalObjectiveScore(param);
            return;
        }

        if (cb.getStructType().intValue() == 1) {
            if (checkSingleChoice(answer.getAnswer(), cb.getChoiceAnswer())) {
                answer.setScore(cb.getScore());
            } else {
                answer.setScore(0.0);
            }
        }
        if (cb.getStructType().intValue() == 2) {
            ExamCacheBean ec = examService.getExamCacheBean(ExamRecordCacheUtil.getExamId(recordId));
            int r = checkMultipleChoice(answer.getAnswer(), cb.getChoiceAnswer());
            if (ec.getObjectiveScorePolicy().equals(ObjectiveScorePolicyEnum.EQUAL)) {// 全对给分
                if (r == 1) {
                    answer.setScore(cb.getScore());
                } else {
                    answer.setScore(0.0);
                }
            }
            if (ec.getObjectiveScorePolicy().equals(ObjectiveScorePolicyEnum.PARTIAL)) {// 漏选半分
                if (r == 1) {
                    answer.setScore(cb.getScore());
                } else if (r == 0) {
                    BigDecimal b = new BigDecimal(cb.getScore()).divide(new BigDecimal("2"), 1, BigDecimal.ROUND_UP);
                    answer.setScore(b.doubleValue());
                } else {
                    answer.setScore(0.0);
                }
            }
        }
        if (cb.getStructType().intValue() == 3) {
            if (StringUtils.isNotBlank(answer.getAnswer())) {
                String answerStr = answer.getAnswer().replaceAll("\n", "");
                if (StringUtils.isNotBlank(answerStr)) {
                    if (Boolean.parseBoolean(answerStr) == cb.getBoolAnswer().booleanValue()) {
                        answer.setScore(cb.getScore());
                    } else {
                        answer.setScore(0.0);
                    }
                }
            }
        }

        if (answer.getScore() == null) {
            log.info("小题分数为空，得分置为0，" + "recordId:" + recordId + " questionKey:" + key);
            answer.setScore(0.0);
        }
        // 更新分数
        redisUtil.set(RedisKeyHelper.examAnswerKey(recordId), key, answer);

        // 计算客观分总分
        calculateTotalObjectiveScore(param);
    }

    @SuppressWarnings("unchecked")
    private void calculateTotalObjectiveScore(Map<String, Object> param) {
        Long recordId = (Long) param.get("recordId");
        Integer mainNumber = (Integer) param.get("mainNumber");
        Integer subNumber = (Integer) param.get("subNumber");
        Integer subIndex = (Integer) param.get("subIndex");
        String lockKey = SystemConstant.REDIS_LOCK_TOTAL_OBJECTIVE_SCORE_PREFIX + recordId;
        try {
            String key = RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex);
            Boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_CACHE_TIME_OUT);
            if (lock) {
                log.info("calculateTotalObjectiveScore get lock sucss recordId:" + recordId + " key:" + key);
                Map<String, ExamStudentAnswerCacheBean> as = redisUtil
                        .getHashEntries(RedisKeyHelper.examAnswerKey(recordId));
                if (as != null && as.size() > 0) {
                    Double total = 0.0;
                    for (ExamStudentAnswerCacheBean sa : as.values()) {
                        if (sa.getObjective() != null && sa.getObjective()) {//是客观题
                            if (sa.getScore() != null) {//有分值
                                total = total + sa.getScore();
                            } else {
                                log.info(
                                        "calculateTotalObjectiveScore ExamStudentAnswerCacheBean Score is null recordId:"
                                                + recordId + " key:" + RedisKeyHelper
                                                .examAnswerHashKey(sa.getMainNumber(), sa.getSubNumber(),
                                                        sa.getSubIndex()));
                                total = null;
                                break;
                            }
                        }
                    }
                    if (total != null) {
                        ExamRecordCacheUtil.setObjectiveScore(recordId, total, true);
                    } else {
                        log.info("calculateTotalObjectiveScore total Score is null recordId:" + recordId + " key:"
                                + key);
                    }
                }
            } else {
                log.info("calculateTotalObjectiveScore get lock faild recordId:" + recordId + " key:" + key);
                calculateObjectiveScoreMsg(recordId, mainNumber, subNumber, subIndex);
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
    }

    //考试中计算客观分消息
    private void calculateObjectiveScoreMsg(Long recordId, Integer mainNumber, Integer subNumber, Integer subIndex) {
        Map<String, Object> transMap = new HashMap<String, Object>();
        transMap.put("recordId", recordId);
        transMap.put("mainNumber", mainNumber);
        transMap.put("subNumber", subNumber);
        transMap.put("subIndex", subIndex);

        String level = "1s";
        Integer time = SystemConstant.mqDelayLevel.get(level);
        LocalDateTime dt = LocalDateTime.now();
        dt = dt.plusSeconds(Long.parseLong(level.replace("s", "")));
        Map<String, Object> propMap = new HashMap<String, Object>();
        propMap.put("timeOut", time);
        propMap.put("mqExecTime", dt.toInstant(ZoneOffset.of("+8")).toEpochMilli());
        MqDto mqDto = new MqDto(MqTopicEnum.THEMIS_TOPIC.getCode(), MqTagEnum.CALCULATE_OBJECTIVE_SCORE.name(),
                transMap, MqTagEnum.CALCULATE_OBJECTIVE_SCORE, recordId.toString(), propMap, recordId.toString());

        mqDtoService.assembleSendAsyncDelayMsg(mqDto);
    }

    private boolean checkSingleChoice(String answerArray, JSONArray ar) {
        JSONArray an = JSONArray.parseArray(answerArray);
        if (an.size() == 0) {
            return false;
        }
        int a = an.getIntValue(0);
        return checkSingleChoice(a, ar);
    }

    private boolean checkSingleChoice(int answer, JSONArray ar) {
        for (int i = 0; i < ar.size(); i++) {
            if (answer == ar.getIntValue(i)) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param answer
     * @param ar
     * @return -1 有错选项，0-无措选项 未答全 1-全对
     */
    private int checkMultipleChoice(String answer, JSONArray ar) {
        int yes = 0;
        int no = 0;
        JSONArray an = JSONArray.parseArray(answer);
        for (int i = 0; i < an.size(); i++) {
            if (checkSingleChoice(an.getIntValue(i), ar)) {
                yes++;
                continue;
            } else {
                no++;
            }
        }
        if (yes == ar.size()) {
            return 1;
        }
        if (yes != 0 && no == 0) {
            return 0;
        }
        return -1;
    }

    /**
     * 考试记录数据持久化
     */
    @SuppressWarnings("unchecked")
    @Transactional
    @Override
    public void saveDataByCache(Long recordId) {
        String tempDir = SystemConstant.TEMP_FILES_DIR;
        String dir = tempDir + "/" + uuid() + "/";
        File dfile = new File(dir);
        try {
            dfile.mkdirs();
            String structFilePath = sdf.format(new Date()) + "/" + uuid() + ".json";
            ExamStudentPaperStructCacheBean struct = (ExamStudentPaperStructCacheBean) redisUtil
                    .get(RedisKeyHelper.studentPaperStructKey(recordId));

            //保存考试记录
            Map<String, Object> record = redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
            TOeExamRecord er = SimpleBeanUtil.mapToObject(record, TOeExamRecord.class);
            er.setStatus(ExamRecordStatusEnum.PERSISTED);
            if (struct != null) {
                er.setPaperStructPath(structFilePath);
            }
            er.setPaperStructUpload(1);
            saveOrUpdate(er);
            //保存作答
            Map<String, ExamStudentAnswerCacheBean> answerMap = redisUtil
                    .getHashEntries(RedisKeyHelper.examAnswerKey(recordId));
            for (ExamStudentAnswerCacheBean answerCache : answerMap.values()) {
                TOeExamAnswer answer = new TOeExamAnswer();
                BeanUtils.copyProperties(answerCache, answer);
                answer.setId(uidUtil.getId());
                answer.setExamRecordId(recordId);
                examAnswerService.saveOrUpdate(answer);
            }
            //            //更新考生信息
            //            ExamStudentCacheBean examStudentCache = examStudentService.getExamStudentCacheBean(er.getExamStudentId());
            //            TEExamStudent examStudent = new TEExamStudent();
            //            BeanUtils.copyProperties(examStudentCache, examStudent);
            //            examStudentService.saveOrUpdate(examStudent);
            //上传个人试卷结构
            if (struct != null) {
                ossUtil.upload(false, structFilePath, struct.getContent());
            }
        } finally {
            FileUtil.deleteFolder(dir);
        }
    }

    private String uuid() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    @Override
    public void saveFaceVerify(ExamTypeEnum type, Long recordId, Long entryAuthenticationId,
            VerifyExceptionEnum entryAuthenticationResult) {
        if (ExamTypeEnum.FIRST_START.equals(type)) {
            ExamRecordCacheUtil.setEntryAuthenticationId(recordId, entryAuthenticationId, false);
            ExamRecordCacheUtil.setEntryAuthenticationResult(recordId, entryAuthenticationResult, false);
            String[] columns = new String[] { ExamRecordFieldEnum.entry_authentication_id.name(),
                    ExamRecordFieldEnum.entry_authentication_result.name() };
            Object[] values = new Object[] { entryAuthenticationId, entryAuthenticationResult };
            this.dataUpdatesMq(recordId, columns, values);
        } else if (ExamTypeEnum.IN_PROCESS.equals(type)) {

        }
    }

    @Override
    public void saveLivenessVerify(LivenessTypeEnum type, Long recordId, Long entryAuthenticationId,
            VerifyExceptionEnum entryAuthenticationResult) {
        if (LivenessTypeEnum.FIRST_START.equals(type)) {
            ExamRecordCacheUtil.setEntryAuthenticationId(recordId, entryAuthenticationId, false);
            ExamRecordCacheUtil.setEntryAuthenticationResult(recordId, entryAuthenticationResult, false);
            String[] columns = new String[] { ExamRecordFieldEnum.entry_authentication_id.name(),
                    ExamRecordFieldEnum.entry_authentication_result.name() };
            Object[] values = new Object[] { entryAuthenticationId, entryAuthenticationResult };
            this.dataUpdatesMq(recordId, columns, values);
        } else if (LivenessTypeEnum.IN_PROCESS.equals(type)) {
            Integer count = ExamRecordCacheUtil.getInProcessLivenessVerifyCount(recordId);
            ExamRecordCacheUtil.setInProcessLivenessVerifyCount(recordId, (count == null ? 0 : count + 1), true);
        }
    }

    @Override
    public void dataUpdateMq(Long recordId, String colName, Object colValue) {
        Map<String, Object> transMap = new HashMap<String, Object>();
        transMap.put("recordId", recordId);
        transMap.put("colName", colName);
        transMap.put("colValue", colValue);
        //mq发送消息start
        MqDto mqDto = new MqDto(MqTopicEnum.THEMIS_TOPIC.getCode(), MqTagEnum.EXAM_RECORD_UPDATE.name(), transMap,
                MqTagEnum.EXAM_RECORD_UPDATE, recordId.toString(), colName);
        mqDtoService.assembleSendOneWayMsg(mqDto);
    }

    /**
     * 单字段更新
     *
     * @param recordId
     * @param colName
     * @param colValue
     */
    @Transactional
    @Override
    public void dataUpdate(Long recordId, String colName, Object colValue) {
        tOeExamRecordMapper.dataUpdate(recordId, colName, colValue);
    }

    /**
     * 多字段更新
     *
     * @param recordId
     * @param colNames
     * @param colValues
     */
    @Override
    @Transactional
    public void dataUpdatesMq(Long recordId, String[] colNames, Object[] colValues) {
        Map<String, Object> transMap = new HashMap<String, Object>();
        transMap.put("recordId", recordId);
        transMap.put("colNames", colNames);
        transMap.put("colValues", colValues);
        //mq发送消息start
        MqDto mqDto = new MqDto(MqTopicEnum.THEMIS_TOPIC.getCode(), MqTagEnum.EXAM_RECORD_UPDATE_COLUMNS.name(),
                transMap, MqTagEnum.EXAM_RECORD_UPDATE_COLUMNS, recordId.toString(), recordId.toString());
        mqDtoService.assembleSendOneWayMsg(mqDto);
    }

    @Override
    @Transactional
    public void dataInitMq(Map<String, Object> param) {
        Long id = (Long) param.get("id");
        //mq发送消息start
        MqDto mqDto = new MqDto(MqTopicEnum.THEMIS_TOPIC.getCode(), MqTagEnum.EXAM_RECORD_INIT.name(), param,
                MqTagEnum.EXAM_RECORD_INIT, id.toString(), id.toString());
        mqDtoService.assembleSendOneWayMsg(mqDto);
    }

    @Transactional
    @Override
    public void dataInit(Map<String, Object> param) {
        TOeExamRecord tr = new TOeExamRecord();
        tr.setId((Long) param.get("id"));
        tr.setExamId((Long) param.get("examId"));
        tr.setExamActivityId((Long) param.get("examActivityId"));
        tr.setExamStudentId((Long) param.get("examStudentId"));
        tr.setPaperId((Long) param.get("paperId"));
        tr.setSerialNumber((Integer) param.get("serialNumber"));
        tr.setFirstPrepareTime((Long) param.get("firstPrepareTime"));
        tr.setStatus(ExamRecordStatusEnum.valueOf((String) param.get("status")));
        tr.setObjectiveScore(0.0);
        tr.setAlreadyBreakCount(0);
        tr.setStartTime((Long) param.get("startTime"));
        tr.setEndTime((Long) param.get("endTime"));
        tr.setOpeningSeconds((Integer) param.get("openingSeconds"));
        tr.setMinDurationSeconds((Integer) param.get("minDurationSeconds"));
        tr.setMaxDurationSeconds((Integer) param.get("maxDurationSeconds"));
        tr.setForceFinish((Integer) param.get("forceFinish"));
        tr.setHasAnswerFile((Integer) param.get("hasAnswerFile"));
        saveOrUpdate(tr);
    }

    /**
     * 查询实时监控台列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param paperDownload
     * @param status
     * @param name
     * @param identity
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param monitorStatusSource
     * @param userId
     * @return
     */
    @Override
    public IPage<InvigilateListBean> invigilatePageList(IPage<Map> iPage, Long examId, Long examActivityId,
            String roomCode, Integer paperDownload, String status, String name, String identity,
            Integer minWarningCount, Integer maxWarningCount, String clientWebsocketStatus, String monitorStatusSource,
            Long userId) {
        return tOeExamRecordMapper
                .invigilatePageList(iPage, examId, examActivityId, roomCode, paperDownload, status, name, identity,
                        minWarningCount, maxWarningCount, clientWebsocketStatus, monitorStatusSource, userId);
    }

    /**
     * 查询实时监控台视频列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param paperDownload
     * @param status
     * @param name
     * @param identity
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param userId
     * @param orgId
     * @return
     */
    @Override
    public IPage<InvigilateListVideoBean> invigilatePageListVideo(IPage<Map> iPage, Long examId, Long examActivityId,
            String roomCode, Integer paperDownload, String status, String name, String identity,
            Integer minWarningCount, Integer maxWarningCount, String clientWebsocketStatus, Long userId, Long orgId) {
        return tOeExamRecordMapper
                .invigilatePageListVideo(iPage, examId, examActivityId, roomCode, paperDownload, status, name, identity,
                        minWarningCount, maxWarningCount, clientWebsocketStatus, userId, orgId);
    }

    /**
     * 实时监控台视频随机列表
     *
     * @param examId
     * @param userId
     * @param randomNum
     * @param orgId
     * @return
     */
    @Override
    public List<InvigilateListVideoBean> invigilatePageListVideoRandom(Long examId, Long userId, Integer randomNum,
            Long orgId) {
        return tOeExamRecordMapper.invigilatePageListVideoRandom(examId, userId, randomNum, orgId);
    }

    /**
     * 查询在线巡考列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param status
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param userId
     * @param orgId
     * @return
     */
    @Override
    public IPage<InvigilateListPatrolBean> invigilatePagePatrolList(IPage<Map> iPage, Long examId, Long examActivityId,
            String roomCode, String status, String name, String identity, Integer minMultipleFaceCount,
            Integer maxMultipleFaceCount, Integer minExceptionCount, Integer maxExceptionCount, Integer minWarningCount,
            Integer maxWarningCount, String clientWebsocketStatus, Long userId, Long orgId) {
        return tOeExamRecordMapper
                .invigilatePagePatrolList(iPage, examId, examActivityId, roomCode, status, name, identity,
                        minMultipleFaceCount, maxMultipleFaceCount, minExceptionCount, maxExceptionCount,
                        minWarningCount, maxWarningCount, clientWebsocketStatus, userId, orgId);
    }

    /**
     * 预警提醒列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param approveStatus
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    @Override
    public IPage<InvigilateListWarningBean> invigilatePageWarningList(IPage<Map> iPage, Long examId,
            Long examActivityId, String roomCode, Integer approveStatus, String name, String identity,
            Integer minMultipleFaceCount, Integer maxMultipleFaceCount, Integer minExceptionCount,
            Integer maxExceptionCount, Integer minWarningCount, Integer maxWarningCount, Long userId, Long orgId) {
        return tOeExamRecordMapper
                .invigilatePageWarningList(iPage, examId, examActivityId, roomCode, approveStatus, name, identity,
                        minMultipleFaceCount, maxMultipleFaceCount, minExceptionCount, maxExceptionCount,
                        minWarningCount, maxWarningCount, userId, orgId);
    }

    /**
     * 预警提醒清除未阅列表
     *
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param approveStatus
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    @Override
    public Integer approveStatusListUpdate(Long examId, Long examActivityId, String roomCode, Integer approveStatus,
            String name, String identity, Integer minMultipleFaceCount, Integer maxMultipleFaceCount,
            Integer minExceptionCount, Integer maxExceptionCount, Integer minWarningCount, Integer maxWarningCount,
            Long userId, Long orgId) {
        return tOeExamRecordMapper
                .approveStatusListUpdate(examId, examActivityId, roomCode, approveStatus, name, identity,
                        minMultipleFaceCount, maxMultipleFaceCount, minExceptionCount, maxExceptionCount,
                        minWarningCount, maxWarningCount, userId, orgId);
    }

    /**
     * 进度查询列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param name
     * @param identity
     * @param userId
     * @return
     */
    @Override
    public IPage<InvigilateListProgressBean> invigilatePageProgressList(IPage<Map> iPage, Long examId,
            Long examActivityId, String roomCode, String courseCode, String name, String identity, Long userId,
            Long orgId) {
        return tOeExamRecordMapper
                .invigilatePageProgressList(iPage, examId, examActivityId, roomCode, courseCode, name, identity, userId,
                        orgId);
    }

    /**
     * 进度查询列表导出
     *
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param name
     * @param identity
     * @param userId
     * @param orgId
     * @return
     */
    @Override
    public List<InvigilateListProgressExcelBean> invigilatePageProgressListExport(Long examId, Long examActivityId,
            String roomCode, String courseCode, String name, String identity, Long userId, Long orgId) {
        return tOeExamRecordMapper
                .invigilatePageProgressListExport(examId, examActivityId, roomCode, courseCode, name, identity, userId,
                        orgId);
    }

    /**
     * 监考明细管理列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param status
     * @param breachStatus
     * @param finishType
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    @Override
    public IPage<InvigilateListHistoryBean> invigilatePageListHistory(IPage<Map> iPage, Long examId,
            Long examActivityId, String roomCode, String courseCode, String status, Integer breachStatus,
            String finishType, String name, String identity, Integer minMultipleFaceCount, Integer maxMultipleFaceCount,
            Integer minExceptionCount, Integer maxExceptionCount, Integer minWarningCount, Integer maxWarningCount,
            Long userId, Long orgId) {
        return tOeExamRecordMapper
                .invigilatePageListHistory(iPage, examId, examActivityId, roomCode, courseCode, status, breachStatus,
                        finishType, name, identity, minMultipleFaceCount, maxMultipleFaceCount, minExceptionCount,
                        maxExceptionCount, minWarningCount, maxWarningCount, userId, orgId);
    }

    /**
     * 重新算分
     */
    @Transactional
    @Override
    public void calculateScore(Long recordId) {

        TOeExamRecord rec = this.getById(recordId);

        ExamPaperCacheBean ep = examPaperService.getExamPaperCacheBean(rec.getPaperId());
        if (rec.getHasAnswerFile().intValue() == 0 && StringUtils.isNotBlank(ep.getAnswerPath())) {
            tOeExamRecordMapper.updateHasAnswerFile(recordId, 1);
        }

        Map<String, ObjectiveAnswerCacheBean> map = examPaperService.getObjectiveAnswerCacheBean(rec.getPaperId());
        if (map == null || map.size() == 0) {
            log.info("no ObjectiveAnswerCacheBean for calculateObjectiveScore recordId:" + recordId);
            return;
        }
        QueryWrapper<TOeExamAnswer> wp = new QueryWrapper<>();
        wp.lambda().eq(TOeExamAnswer::getExamRecordId, recordId);
        List<TOeExamAnswer> answers = examAnswerService.list(wp);
        if (answers == null || answers.size() == 0) {
            log.info("no ObjectiveAnswer for calculateObjectiveScore recordId:" + recordId);
            return;
        }
        BigDecimal tatolscore = new BigDecimal(0.0);
        for (TOeExamAnswer answer : answers) {
            Double score = calculateScoreForQuestion(answer, map);
            if (score != null) {
                tatolscore = tatolscore.add(new BigDecimal(score));
            }
        }

        // 设置客观分总分
        tOeExamRecordMapper.updateObjectiveScore(recordId, tatolscore.doubleValue());
    }

    private Double calculateScoreForQuestion(TOeExamAnswer answer, Map<String, ObjectiveAnswerCacheBean> map) {
        Long recordId = answer.getExamRecordId();
        Integer mainNumber = answer.getMainNumber();
        Integer subNumber = answer.getSubNumber();
        Integer subIndex = answer.getSubIndex();
        String key = RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex);
        ObjectiveAnswerCacheBean cb = map.get(key);
        if (cb == null) {
            log.info("not a objective question  recordId:" + answer.getExamRecordId() + " key:" + key);
            return null;
        }

        if (cb.getStructType().intValue() == 1) {
            if (checkSingleChoice(answer.getAnswer(), cb.getChoiceAnswer())) {
                answer.setScore(cb.getScore());
            } else {
                answer.setScore(0.0);
            }
        }
        if (cb.getStructType().intValue() == 2) {
            ExamCacheBean ec = examService.getExamCacheBean(tOeExamRecordMapper.selectById(recordId).getExamId());
            int r = checkMultipleChoice(answer.getAnswer(), cb.getChoiceAnswer());
            if (ec.getObjectiveScorePolicy().equals(ObjectiveScorePolicyEnum.EQUAL)) {// 全对给分
                if (r == 1) {
                    answer.setScore(cb.getScore());
                } else {
                    answer.setScore(0.0);
                }
            }
            if (ec.getObjectiveScorePolicy().equals(ObjectiveScorePolicyEnum.PARTIAL)) {// 漏选半分
                if (r == 1) {
                    answer.setScore(cb.getScore());
                } else if (r == 0) {
                    BigDecimal b = new BigDecimal(cb.getScore()).divide(new BigDecimal("2"), 1, BigDecimal.ROUND_UP);
                    answer.setScore(b.doubleValue());
                } else {
                    answer.setScore(0.0);
                }
            }
        }
        if (cb.getStructType().intValue() == 3) {
            if (StringUtils.isNotBlank(answer.getAnswer())) {
                String answerStr = answer.getAnswer().replaceAll("\n", "");
                if (StringUtils.isNotBlank(answerStr)) {
                    if (Boolean.parseBoolean(answerStr) == cb.getBoolAnswer().booleanValue()) {
                        answer.setScore(cb.getScore());
                    } else {
                        answer.setScore(0.0);
                    }
                }
            }
        }

        examAnswerService.saveOrUpdate(answer);

        return answer.getScore();
    }

    /**
     * 获取考试记录数
     *
     * @param examId
     * @return
     */
    @Override
    public Long getCountByExamId(Long examId) {
        return tOeExamRecordMapper.getCountByExamId(examId);
    }

    /**
     * 批量获取考试记录
     *
     * @param examId
     * @param startId
     * @return
     */
    @Override
    public List<TOeExamRecord> getListByExamIdAndStartId(Long examId, Long startId) {
        return tOeExamRecordMapper.getListByExamIdAndStartId(examId, startId);
    }

    @Override
    public TOeExamRecord findOneByPaperId(Long paperId) {
        return tOeExamRecordMapper.findOneByPaperId(paperId);
    }

    @Override
    public TOeExamRecord findOneByExamId(Long examId, ExamRecordStatusEnum status) {
        return tOeExamRecordMapper.findOneByExamId(examId, status == null ? null : status.name());
    }

    @Override
    public List<OpenRecordNeedMarkBean> findExamRecordNeedMark(Long examId, List<Long> examStudentIds) {
        return tOeExamRecordMapper.findExamRecordNeedMark(examId, examStudentIds);
    }

    /**
     * 寻找客观分最高
     *
     * @param examId
     * @param examStudentId
     * @return
     */
    @Override
    public List<TOeExamRecord> findMaxObjectiveScore(Long examId, Long examStudentId) {
        return tOeExamRecordMapper.findMaxObjectiveScore(examId, examStudentId);
    }

    /**
     * 查询成绩
     *
     * @param markResultDto
     * @param recordSelectStrategyEnum
     * @return
     */
    @Override
    public TOeExamRecord findMarkResult(MarkResultDto markResultDto,
            RecordSelectStrategyEnum recordSelectStrategyEnum) {
        TOeExamRecord tOeExamRecord = null;
        Double sumScore = null;
        //客观分最高
        if (Objects.nonNull(recordSelectStrategyEnum) && Objects
                .equals(recordSelectStrategyEnum, RecordSelectStrategyEnum.HIGHEST_OBJECTIVE_SCORE)) {
            List<TOeExamRecord> tOeExamRecordList = this
                    .findMaxObjectiveScore(markResultDto.getExamId(), markResultDto.getExamStudentId());
            if (Objects.nonNull(tOeExamRecordList) && tOeExamRecordList.size() > 0) {
                Long paperId = tOeExamRecordList.get(0).getPaperId();
                ExamPaperCacheBean paperCacheBean = examPaperService.getExamPaperCacheBean(paperId);
                boolean isHasSubjective = true;
                if (paperCacheBean.getTotalSubjectiveScore().intValue() == 0) {
                    isHasSubjective = false;
                }
                //是否有主观题
                if (!isHasSubjective) {
                    for (TOeExamRecord t : tOeExamRecordList) {
                        if (Objects.isNull(tOeExamRecord)) {
                            tOeExamRecord = t;
                        } else {
                            if (Objects.nonNull(t.getObjectiveScore()) && Objects
                                    .nonNull(tOeExamRecord.getObjectiveScore())
                                    && t.getObjectiveScore().doubleValue() >= tOeExamRecord.getObjectiveScore()
                                    .doubleValue() && Objects.nonNull(t.getFinishTime()) && Objects
                                    .nonNull(tOeExamRecord.getFinishTime())
                                    && t.getFinishTime().longValue() > tOeExamRecord.getFinishTime().longValue()) {
                                tOeExamRecord = t;
                            }
                        }
                    }
                } else {
                    //总分最高选取
                    tOeExamRecord = getMaxSumScore(tOeExamRecord, sumScore, markResultDto);
                }
            }
        }
        //最后一次提交
        else if (Objects.nonNull(recordSelectStrategyEnum) && Objects
                .equals(recordSelectStrategyEnum, RecordSelectStrategyEnum.LATEST)) {
            List<TOeExamRecord> tOeExamRecordList = this
                    .findMaxObjectiveScore(markResultDto.getExamId(), markResultDto.getExamStudentId());
            for (TOeExamRecord t : tOeExamRecordList) {
                if (Objects.isNull(tOeExamRecord)) {
                    tOeExamRecord = t;
                } else {
                    if (Objects.nonNull(t.getFinishTime()) && Objects.nonNull(tOeExamRecord.getFinishTime())
                            && t.getFinishTime().longValue() > tOeExamRecord.getFinishTime().longValue()) {
                        tOeExamRecord = t;
                    }
                }
            }
        }
        //全部阅卷后取最高分
        else if (Objects.nonNull(recordSelectStrategyEnum) && Objects
                .equals(recordSelectStrategyEnum, RecordSelectStrategyEnum.HIGHEST_TOTAL_SCORE)) {
            tOeExamRecord = getMaxSumScore(tOeExamRecord, sumScore, markResultDto);
        }
        return tOeExamRecord;
    }

    /**
     * 设置断点信息
     *
     * @param recordId
     */
    @Override
    @Transactional
    public void setExamBreak(Long recordId) {
        if (redisUtil.lock(SystemConstant.REDIS_LOCK_EXAM_BREAK_PREFIX + recordId,
                SystemConstant.REDIS_LOCK_EXAM_BREAK_TIME_OUT)) {
            try {
                Integer alreadyBreakCount = Objects.isNull(ExamRecordCacheUtil.getAlreadyBreakCount(recordId)) ?
                        0 :
                        ExamRecordCacheUtil.getAlreadyBreakCount(recordId);
                alreadyBreakCount++;
                Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
                Long breakId = uidUtil.getId();
                ExamRecordCacheUtil.setLastBreakId(recordId, breakId, false);
                ExamRecordCacheUtil.setStatus(recordId, ExamRecordStatusEnum.BREAK_OFF, false);
                Long lastBreakTimeNow = System.currentTimeMillis();
                ExamRecordCacheUtil.setLastBreakTime(recordId, lastBreakTimeNow, false);
                ExamRecordCacheUtil.setAlreadyBreakCount(recordId, alreadyBreakCount, false);
                Long lastStartTime = System.currentTimeMillis();
                ExamRecordCacheUtil.setLastStartTime(recordId, lastStartTime, false);
                String[] columns = new String[] { ExamRecordFieldEnum.last_break_id.name(),
                        ExamRecordFieldEnum.status.name(), ExamRecordFieldEnum.last_break_time.name(),
                        ExamRecordFieldEnum.already_break_count.name(), ExamRecordFieldEnum.last_start_time.name() };
                Object[] values = new Object[] { breakId, ExamRecordStatusEnum.BREAK_OFF, lastBreakTimeNow,
                        alreadyBreakCount, lastStartTime };
                TOeExamRecordService tOeExamRecordService = SpringContextHolder.getBean(TOeExamRecordService.class);
                tOeExamRecordService.dataUpdatesMq(recordId, columns, values);
                //考试断点异常原因 发送mq start
                MqDto mqDtoBreak = new MqDto(MqTopicEnum.THEMIS_TOPIC.getCode(), MqTagEnum.EXAM_BREAK.name(),
                        ExceptionEnum.NET_TIME_OUT, MqTagEnum.EXAM_BREAK, String.valueOf(recordId),
                        String.valueOf(recordId));
                MqDtoService mqDtoService = SpringContextHolder.getBean(MqDtoService.class);
                mqDtoService.assembleSendOneWayMsg(mqDtoBreak);
                //考试断点异常原因 发送mq end

                Long examId = ExamRecordCacheUtil.getExamId(recordId);
                ExamCacheBean ec = examService.getExamCacheBean(examId);//考试缓存
                Integer breakExpireSeconds = Objects.isNull(ec.getBreakExpireSeconds()) ?
                        0 :
                        ec.getBreakExpireSeconds();
                if (breakExpireSeconds > 0) {
                    List<String> list = SystemConstant.mqDelayLevelList.subList(5, 15);
                    String level = null;
                    if (breakExpireSeconds <= 60) {
                        level = "1m";
                    } else {
                        Integer time = breakExpireSeconds.intValue() / 60;
                        if (time >= 30) {
                            level = "30m";
                        } else {
                            for (String s : list) {
                                Integer value = Integer.parseInt(s.substring(0, s.length() - 1));
                                if (time.intValue() <= value.intValue()) {
                                    level = value + "m";
                                    break;
                                }
                            }
                        }
                    }
                    Map<String, Object> tranMap = new HashMap<>();
                    Integer time = SystemConstant.mqDelayLevel.get(level);
                    LocalDateTime dt = LocalDateTime.now();
                    if (level.contains("m")) {
                        dt = dt.plusMinutes(Long.parseLong(level.replace("m", "")));
                    } else {
                        dt = dt.plusSeconds(Long.parseLong(level.replace("s", "")));
                    }
                    tranMap.put("recordId", recordId);
                    tranMap.put("timeOut", time);
                    tranMap.put("mqExecTime", dt.toInstant(ZoneOffset.of("+8")).toEpochMilli());
                    //考试断点延时消息 发送mq start
                    MqDto mqDtoBreakDelay = new MqDto(MqTopicEnum.THEMIS_TOPIC.getCode(),
                            MqTagEnum.EXAM_BREAK_DELAY.name(), MqTagEnum.EXAM_BREAK_DELAY, MqTagEnum.EXAM_BREAK_DELAY,
                            String.valueOf(recordId), tranMap, String.valueOf(recordId));
                    mqDtoService.assembleSendAsyncDelayMsg(mqDtoBreakDelay);
                    //考试断点延时消息 发送mq end
                }
                //更新场次-考试记录缓存
                ExamActivityRecordCacheUtil.setExamRecordStatus(examActivityId, recordId,
                        new ExamActivityRecordCacheBean(ExamRecordCacheUtil.getExamStudentId(recordId),
                                ExamRecordCacheUtil.getStatus(recordId)));
            } catch (Exception e) {
                log.error("请求出错", e);
                if (e instanceof BusinessException) {
                    throw new BusinessException(e.getMessage());
                } else {
                    throw new RuntimeException(e);
                }
            } finally {
                if (Objects.nonNull(recordId)) {
                    redisUtil.releaseLock(SystemConstant.REDIS_LOCK_EXAM_BREAK_PREFIX + recordId);
                }
            }
        }
    }

    /**
     * 发送断点信息
     *
     * @param recordId
     * @param setBreak
     * @return
     */
    @Override
    public Boolean sendExamBreakMsg(Long recordId, boolean setBreak) {
        Boolean finished = false;
        if (redisUtil.lock(SystemConstant.REDIS_LOCK_EXAM_BREAK_LOGIC_PREFIX + recordId,
                SystemConstant.REDIS_LOCK_EXAM_BREAK_LOGIC_TIME_OUT)) {
            try {
                Long examId = ExamRecordCacheUtil.getExamId(recordId);
                Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
                ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
                ExamCacheBean ec = examService.getExamCacheBean(examId);//考试缓存
                Long lastBreakTime = ExamRecordCacheUtil.getLastBreakTime(recordId);
                Integer breakExpireSeconds = Objects.isNull(ec.getBreakExpireSeconds()) ?
                        0 :
                        ec.getBreakExpireSeconds();
                Integer durationSeconds = Objects.isNull(ExamRecordCacheUtil.getDurationSeconds(recordId)) ?
                        0 :
                        ExamRecordCacheUtil.getDurationSeconds(recordId);
                Integer alreadyBreakCount = Objects.isNull(ExamRecordCacheUtil.getAlreadyBreakCount(recordId)) ?
                        0 :
                        ExamRecordCacheUtil.getAlreadyBreakCount(recordId);
                if (setBreak) {//如果需要断点，则次数本地先加1，可以避免多生成考试记录
                    alreadyBreakCount++;
                }
                Integer leftBreakResumeCount = ec.getBreakResumeCount() - alreadyBreakCount;
                if (Objects.nonNull(lastBreakTime)
                        && (System.currentTimeMillis() - lastBreakTime) / 1000 >= breakExpireSeconds) {
                    finished = true;
                    examService.finish(examStudentCacheBean.getStudentId(), recordId, FinishTypeEnum.AUTO.name(),
                            durationSeconds);
                } else {
                    if (leftBreakResumeCount < 0) {
                        finished = true;
                        examService.finish(examStudentCacheBean.getStudentId(), recordId, FinishTypeEnum.AUTO.name(),
                                durationSeconds);
                    } else {
                        if (setBreak) {
                            this.setExamBreak(recordId);
                        }
                    }
                }
            } catch (Exception e) {
                log.error("请求出错", e);
                if (e instanceof BusinessException) {
                    throw new BusinessException(e.getMessage());
                } else {
                    throw new RuntimeException(e);
                }
            } finally {
                if (Objects.nonNull(recordId)) {
                    redisUtil.releaseLock(SystemConstant.REDIS_LOCK_EXAM_BREAK_LOGIC_PREFIX + recordId);
                }
            }
        }
        return finished;
    }

    /**
     * 取最高分
     *
     * @param tOeExamRecord
     * @param sumScore
     * @param markResultDto
     * @return
     */
    private TOeExamRecord getMaxSumScore(TOeExamRecord tOeExamRecord, Double sumScore, MarkResultDto markResultDto) {
        Long examId = null, examStudentId = null;
        if (Objects.isNull(sumScore)) {
            sumScore = markResultDto.getSumScore();
            examId = markResultDto.getExamId();
            examStudentId = markResultDto.getExamStudentId();
        } else {
            if (Objects.nonNull(markResultDto.getSumScore()) && markResultDto.getSumScore().doubleValue() >= sumScore
                    .doubleValue()) {
                sumScore = markResultDto.getSumScore();
                examId = markResultDto.getExamId();
                examStudentId = markResultDto.getExamStudentId();
            }
        }
        if (Objects.nonNull(sumScore)) {
            List<TOeExamRecord> tOeExamRecordList = this.findMaxObjectiveScore(examId, examStudentId);
            if (Objects.nonNull(tOeExamRecordList) && tOeExamRecordList.size() > 0) {
                tOeExamRecord = tOeExamRecordList.get(0);
            }
        }
        return tOeExamRecord;
    }
}
